@@ -1,5 +1,5 @@
 /* dircolors - output commands to set the LS_COLOR environment variable
-   Copyright (C) 1996-2005 Free Software Foundation, Inc.
+   Copyright (C) 1996-2006 Free Software Foundation, Inc.
    Copyright (C) 1994, 1995, 1997, 1998, 1999, 2000 H. Peter Anvin
 
    This program is free software; you can redistribute it and/or modify
@@ -62,7 +62,6 @@ enum Shell_syntax
    variable.  */
 static struct obstack lsc_obstack;
 
-/* FIXME: associate with ls_codes? */
 static const char *const slack_codes[] =
 {
   "NORMAL", "NORM", "FILE", "DIR", "LNK", "LINK",
@@ -75,9 +74,11 @@ static const char *const slack_codes[] =
 static const char *const ls_codes[] =
 {
   "no", "no", "fi", "di", "ln", "ln", "ln", "or", "mi", "pi", "pi",
-  "so", "bd", "bd", "cd", "cd", "do", "ex", "lc", "lc", "rc", "rc", "ec", "ec"
+  "so", "bd", "bd", "cd", "cd", "do", "ex", "lc", "lc", "rc", "rc", "ec", "ec",
   "su", "su", "sg", "sg", "st", "ow", "ow", "tw", "tw", NULL
 };
+#define array_len(Array) (sizeof (Array) / sizeof *(Array))
+verify (array_len (slack_codes) == array_len (ls_codes));
 
 static struct option const long_options[] =
   {
@@ -136,7 +137,7 @@ guess_shell_syntax (void)
   if (shell == NULL || *shell == '\0')
     return SHELL_SYNTAX_UNKNOWN;
 
-  shell = base_name (shell);
+  shell = last_component (shell);
 
   if (STREQ (shell, "csh") || STREQ (shell, "tcsh"))
     return SHELL_SYNTAX_C;
@@ -154,8 +155,8 @@ parse_line (char const *line, char **keyword, char **arg)
   *keyword = NULL;
   *arg = NULL;
 
-  for (p = line; ISSPACE (to_uchar (*p)); ++p)
-    ;
+  for (p = line; isspace (to_uchar (*p)); ++p)
+    continue;
 
   /* Ignore blank lines and shell-style comments.  */
   if (*p == '\0' || *p == '#')
@@ -163,7 +164,7 @@ parse_line (char const *line, char **keyword, char **arg)
 
   keyword_start = p;
 
-  while (!ISSPACE (to_uchar (*p)) && *p != '\0')
+  while (!isspace (to_uchar (*p)) && *p != '\0')
     {
       ++p;
     }
@@ -176,7 +177,7 @@ parse_line (char const *line, char **keyword, char **arg)
     {
       ++p;
     }
-  while (ISSPACE (to_uchar (*p)));
+  while (isspace (to_uchar (*p)));
 
   if (*p == '\0' || *p == '#')
     return;
@@ -186,10 +187,8 @@ parse_line (char const *line, char **keyword, char **arg)
   while (*p != '\0' && *p != '#')
     ++p;
 
-  for (--p; ISSPACE (to_uchar (*p)); --p)
-    {
-      /* empty */
-    }
+  for (--p; isspace (to_uchar (*p)); --p)
+    continue;
   ++p;
 
   *arg = xstrndup (arg_start, p - arg_start);
@@ -207,6 +206,13 @@ append_quoted (const char *str)
     {
       switch (*str)
 	{
+	case '\'':
+	  APPEND_CHAR ('\'');
+	  APPEND_CHAR ('\\');
+	  APPEND_CHAR ('\'');
+	  need_backslash = true;
+	  break;
+
 	case '\\':
 	case '^':
 	  need_backslash = !need_backslash;
